package de.unikoblenz.fgbks.base.utils.boundary.checker;

import de.unikoblenz.fgbks.base.utils.boundary.impl.StringBoundary;

public class CheckStringSubsumes extends AbstractBoundaryCheck<StringBoundary> {

  private static CheckStringSubsumes instance = new CheckStringSubsumes();

  public static CheckStringSubsumes getInstance() {
    return instance;
  }

  @Override
  public BoundaryCheckType getType() {
    return BoundaryCheckType.IS_NOT_IN_CONTACT;
  }

  @Override
  public boolean check(StringBoundary b1, StringBoundary b2) {
    return checkSubsumes(b1, b2);
  }

  public static boolean checkSubsumes(StringBoundary b1, StringBoundary b2) {
    if (b1.isMatchAny() && !b2.isMatchAny()) {
      return true;
    }
    if (b2.isMatchAny()) {
      return false;
    }
    int[] b1h = b1.getValuesHashes();
    int[] b2h = b2.getValuesHashes();
    // not() and not not()
    if (b1.matchesNoneOfValues() && !b2.matchesNoneOfValues()) {
      for (int i = 0; i < b1h.length; i++) {
        for (int u = 0; u < b2h.length; u++) {
          if (b1h[i] == b2h[u]) {
            return false;
          }
        }
      }
      return true;
    }
    // both not()
    if (b1.matchesNoneOfValues() && b2.matchesNoneOfValues()) {
      return checkX(b1h, b2h, b1h.length >= b2h.length);
    }
    // both normal
    return checkX(b2h, b1h, b1h.length <= b2h.length);
  }

  private static boolean checkX(int[] b1h, int[] b2h, boolean b) {
    if (b) {
      return false;
    }
    for (int i : b1h) {
      boolean found = false;
      for (int u : b2h) {
        found |= i == u;
        if (found) {
          break;
        }
      }
      if (!found) {
        return false;
      }
    }
    return true;
  }
}
